@@ -1,7 +1,8 @@
 # debian/rules -- lintian check script -*- perl -*-
 
-# Copyright (C) 2006 Russ Allbery <rra@debian.org>
-# Copyright (C) 2005 René van Bevern <rvb@pro-linux.de>
+# Copyright © 2006 Russ Allbery <rra@debian.org>
+# Copyright © 2005 René van Bevern <rvb@pro-linux.de>
+# Copyright © 2019-2020 Chris Lamb <lamby@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,15 +26,13 @@ use List::MoreUtils qw(any none);
 
 use Lintian::Data;
 use Lintian::SlidingWindow;
-use Lintian::Util qw(rstrip);
 
 use Moo;
 use namespace::clean;
 
 with 'Lintian::Check';
 
-our $PYTHON_DEPEND
-  = 'python:any | python-dev:any | python-all:any | python-all-dev:any | python2:any | python2-dev:any';
+our $PYTHON_DEPEND= 'python2:any | python2-dev:any';
 our $PYTHON3_DEPEND
   = 'python3:any | python3-dev:any | python3-all:any | python3-all-dev:any';
 our $PYTHON2X_DEPEND = 'python2.7:any | python2.7-dev:any';
@@ -41,6 +40,8 @@ our $PYTHON3X_DEPEND= join(' | ',
     map { "python${_}:any | python${_}-dev:any" } qw(3.4 3.5 3.6 3.7));
 our $ANYPYTHON_DEPEND
   = "$PYTHON_DEPEND | $PYTHON2X_DEPEND | $PYTHON3_DEPEND | $PYTHON3X_DEPEND";
+our $PYTHON3_ALL_DEPEND
+  = 'python3-all:any | python3-all-dev:any | python3-all-dbg:any';
 
 my $KNOWN_MAKEFILES = Lintian::Data->new('rules/known-makefiles', '\|\|');
 my $DEPRECATED_MAKEFILES = Lintian::Data->new('rules/deprecated-makefiles');
@@ -154,14 +155,14 @@ sub source {
     # (dpkg-source does as well) and skip all the tests if we then
     # can't read it.
     if ($rules->is_symlink) {
-        $self->tag('debian-rules-is-symlink');
+        $self->hint('debian-rules-is-symlink');
         return unless $rules->is_open_ok;
     }
 
-    my $architecture = $processable->field('architecture', '');
-    my $version = $processable->field('version');
+    my $architecture = $processable->fields->value('Architecture');
+
     # If the version field is missing, we assume a neutral non-native one.
-    $version = '0-1' unless defined $version;
+    my $version = $processable->fields->value('Version') || '0-1';
 
     open(my $rules_fd, '<', $rules->unpacked_path);
 
@@ -169,11 +170,11 @@ sub source {
     # strict reading of Policy doesn't allow either, but they seem harmless.
     my $start = <$rules_fd>;
     $start //= q{};
-    $self->tag('debian-rules-not-a-makefile')
+    $self->hint('debian-rules-not-a-makefile')
       unless $start =~ m%^\#!\s*/usr/bin/make\s+-[re]?f[re]?\s*$%;
 
     # Check if debian/rules is marked as executable.
-    $self->tag('debian-rules-not-executable') unless $rules->is_executable;
+    $self->hint('debian-rules-not-executable') unless $rules->is_executable;
 
     # Holds which dependencies are required.  The keys in %needed and
     # %needed_clean are the dependencies; the values are the tags to use or the
@@ -189,7 +190,7 @@ sub source {
     # other files, since to chase all includes we'd have to have all
     # of its build dependencies installed.
     local $_;
-    my $build_all = $processable->relation('build-depends-all');
+    my $build_all = $processable->relation('Build-Depends-All');
     my @arch_rules = (qr/^clean$/, qr/^binary-arch$/, qr/^build-arch$/);
     my @indep_rules = (qr/^build$/, qr/^build-indep$/, qr/^binary-indep$/);
     my (@current_targets, %rules_per_target,  %debhelper_group);
@@ -204,22 +205,22 @@ sub source {
         }
         my $line = $_;
 
-        $self->tag('debian-rules-is-dh_make-template')
+        $self->hint('debian-rules-is-dh_make-template')
           if $line =~ m/dh_make generated override targets/;
 
         next if /^\s*\#/;
-        if (m/^\s*[s-]?include\s+(\S++)/o){
+        if (/^\s*[s-]?include\s+(\S++)/){
             my $makefile = $1;
             my $targets = $KNOWN_MAKEFILES->value($makefile);
             if (defined $targets){
-                foreach my $target (split m/\s*+,\s*+/o, $targets){
+                foreach my $target (split /\s*+,\s*+/, $targets){
                     $seen{$target}++ if $POLICYRULES->known($target);
                 }
             } else {
                 $includes = 1;
             }
             if ($DEPRECATED_MAKEFILES->known($makefile)){
-                $self->tag('debian-rules-uses-deprecated-makefile',
+                $self->hint('debian-rules-uses-deprecated-makefile',
                     "line $.",$makefile);
             }
         }
@@ -228,7 +229,7 @@ sub source {
         # deprecated.  It's a bit easier structurally to do this here than in
         # debhelper.
         if (/^\s*(?:export\s+)?DH_COMPAT\s*:?=/ && keys(%seen) == 0) {
-            $self->tag('debian-rules-sets-DH_COMPAT', "line $.");
+            $self->hint('debian-rules-sets-DH_COMPAT', "line $.");
         }
 
         # Check for problems that can occur anywhere in debian/rules.
@@ -237,12 +238,12 @@ sub source {
         ) {
             # Ignore "-C<dir>" (#671537)
             if (not $1 or $1 !~ m,^C,) {
-                $self->tag('debian-rules-ignores-make-clean-error', "line $.");
+                $self->hint('debian-rules-ignores-make-clean-error',"line $.");
             }
         }
 
         if (/^\s*(?:export\s+)?DEB_BUILD_OPTIONS\s*:?=/ && keys(%seen) == 0) {
-            $self->tag('debian-rules-sets-DEB_BUILD_OPTIONS', "line $.");
+            $self->hint('debian-rules-sets-DEB_BUILD_OPTIONS', "line $.");
         }
 
         if (
@@ -252,7 +253,7 @@ sub source {
             /x
             && keys(%seen) == 0
         ) {
-            $self->tag('debian-rules-sets-dpkg-architecture-variable',
+            $self->hint('debian-rules-sets-dpkg-architecture-variable',
                 "$1 (line $.)");
         }
 
@@ -261,19 +262,23 @@ sub source {
             my $badregex = $BAD_CONSTRUCT_IN_RULES->value($bad_construct);
             if ($line =~ m/$badregex/) {
                 if (defined($+{info})) {
-                    $self->tag($bad_construct, $+{info}, "(line $.)");
+                    $self->hint($bad_construct, $+{info}, "(line $.)");
                 } else {
-                    $self->tag($bad_construct, "line $.");
+                    $self->hint($bad_construct, "line $.");
                 }
             }
         }
 
+        if ($line =~ m/--as-needed/ and $line !~ m/--no-as-needed/) {
+            $self->hint('debian-rules-uses-as-needed-linker-flag',"line $.");
+        }
+
         #<<< no Perl tidy
-        $self->tag(
+        $self->hint(
             'debian-rules-uses-supported-python-versions-without-python-all-build-depends',
             $1, "(line $.)"
         ) if $line =~ m/(py3versions\s+([\w\-\s]*--supported|-\w*s\w*))/
-          and not $build_all->implies('python3-all');
+          and not $build_all->implies($PYTHON3_ALL_DEPEND);
         #>>>
 
         # General assignment - save the variable
@@ -282,7 +287,7 @@ sub source {
             # rather well.
             my ($var, $value) = ($1, $2);
             $variables{$var} = $value;
-            $self->tag('unnecessary-source-date-epoch-assignment', "(line $.)")
+            $self->hint('unnecessary-source-date-epoch-assignment',"(line $.)")
               if $var eq 'SOURCE_DATE_EPOCH'
               and not $build_all->implies(
                 'dpkg-dev (>= 1.18.8) | debhelper (>= 10.10)');
@@ -325,7 +330,9 @@ sub source {
                         # we think we know what it will expand to - note
                         # we ought to "delay" it was a "=" variable rather
                         # than ":=" or "+=".
-                        for (split m/\s++/o, rstrip($val)) {
+
+                   # discards empty elements at end, effectively trimming right
+                        for (split(/\s+/, $val)) {
                             $seen{$_}++ if $POLICYRULES->known($_);
                         }
                         last;
@@ -339,7 +346,7 @@ sub source {
 
         if (!$includes
             && m/dpkg-parsechangelog.*(?:Source|Version|Date|Timestamp)/s) {
-            $self->tag('debian-rules-parses-dpkg-parsechangelog', "(line $.)");
+            $self->hint('debian-rules-parses-dpkg-parsechangelog',"(line $.)");
         }
 
         if (!/^ifn?(?:eq|def)\s/ && m/^([^\s:][^:]*):+(.*)/s) {
@@ -352,7 +359,7 @@ sub source {
             } split(' ', $target_dependencies);
             for my $target (@current_targets) {
                 $overridden{$1} = $. if $target =~ m/override_(.+)/;
-                if ($target =~ m/%/o) {
+                if ($target =~ /%/) {
                     my $pattern = quotemeta $target;
                     $pattern =~ s/\\%/.*/g;
                     for my $rulebypolicy ($POLICYRULES->all) {
@@ -368,7 +375,9 @@ sub source {
                             # we ought to "delay" it was a "=" variable rather
                             # than ":=" or "+=".
                             local $_;
-                            for (split m/\s++/o, rstrip($val)) {
+
+                   # discards empty elements at end, effectively trimming right
+                            for (split(/\s+/, $val)) {
                                 $seen{$_}++ if $POLICYRULES->known($_);
                             }
                             last;
@@ -419,7 +428,8 @@ sub source {
                     my $group = $debhelper_order{$command};
                     $debhelper_group{$package} ||= 0;
                     if ($group < $debhelper_group{$package}) {
-                        $self->tag('debian-rules-calls-debhelper-in-odd-order',
+                        $self->hint(
+                            'debian-rules-calls-debhelper-in-odd-order',
                             $command, "(line $.)");
                     } else {
                         $debhelper_group{$package} = $group;
@@ -436,9 +446,10 @@ sub source {
             unless ($seen{$target}) {
                 my $typerule = $POLICYRULES->value($target);
                 if($typerule eq 'required') {
-                    $self->tag('debian-rules-missing-required-target',$target);
+                    $self->hint('debian-rules-missing-required-target',
+                        $target);
                 } elsif ($typerule eq 'recommended_allindep') {
-                    $self->tag('debian-rules-missing-recommended-target',
+                    $self->hint('debian-rules-missing-recommended-target',
                         $target);
                 } else {
                     $typerule ||= '<N/A>';
@@ -462,13 +473,13 @@ sub source {
                 $nonempty = 1;
             }
         }
-        $self->tag('binary-arch-rules-but-pkg-is-arch-indep') if $nonempty;
+        $self->hint('binary-arch-rules-but-pkg-is-arch-indep') if $nonempty;
     }
 
     foreach my $cmd (qw(dh_clean dh_fixperms)) {
         foreach my $suffix ('', '-indep', '-arch') {
             my $line = $overridden{"$cmd$suffix"};
-            $self->tag("override_$cmd-does-not-call-$cmd", "(line $line)")
+            $self->hint("override_$cmd-does-not-call-$cmd", "(line $line)")
               if $line
               and none { m/^\t\s*-?($cmd\b|\$\(overridden_command\))/ }
             @{$rules_per_target{"override_$cmd$suffix"}};
@@ -483,44 +494,45 @@ sub source {
               and $_
               !~ m{^\t\s*[-@]?(?:(?:/usr)?/bin/)?(?:cp|chmod|echo|ln|mv|mkdir|rm|test|true)}
         } @{$rules_per_target{'override_dh_auto_test'}};
-        $self->tag('override_dh_auto_test-does-not-check-DEB_BUILD_OPTIONS',
+        $self->hint('override_dh_auto_test-does-not-check-DEB_BUILD_OPTIONS',
             "(line $line)")
           if @lines and none { m/(DEB_BUILD_OPTIONS|nocheck)/ } @conditionals;
     }
 
-    $self->tag('debian-rules-contains-unnecessary-get-orig-source-target')
+    $self->hint('debian-rules-contains-unnecessary-get-orig-source-target')
       if any { m/^\s+uscan\b/ } @{$rules_per_target{'get-orig-source'}};
 
     # Make sure that all the required build dependencies are there.  Don't
     # issue missing-build-dependency errors for debhelper, since there's
     # another test that does that and it would just be a duplicate.
-    my $build_regular = $processable->relation('build-depends');
-    my $build_indep   = $processable->relation('build-depends-indep');
+    my $build_regular = $processable->relation('Build-Depends');
+    my $build_indep   = $processable->relation('Build-Depends-Indep');
     for my $package (keys %needed_clean) {
         delete $needed{$package};
         my $tag = $needed_clean{$package} || 'missing-build-dependency';
         unless ($build_regular->implies($package)) {
             if ($build_indep->implies($package)) {
-                $self->tag('clean-should-be-satisfied-by-build-depends',
+                $self->hint(
+                    'missing-build-depends-for-clean-target-in-debian-rules',
                     $package);
             } else {
                 if ($tag eq 'missing-build-dependency') {
-                    $self->tag($tag, $package) if $package ne 'debhelper';
+                    $self->hint($tag, $package) if $package ne 'debhelper';
                 } else {
-                    $self->tag($tag);
+                    $self->hint($tag);
                 }
             }
         }
     }
-    my $noarch = $processable->relation_noarch('build-depends-all');
+    my $noarch = $processable->relation_noarch('Build-Depends-All');
     for my $package (keys %needed) {
         my $tag = $needed{$package} || 'missing-build-dependency';
 
         unless ($noarch->implies($package)) {
             if ($tag eq 'missing-build-dependency') {
-                $self->tag($tag, $package);
+                $self->hint($tag, $package);
             } else {
-                $self->tag($tag);
+                $self->hint($tag);
             }
         }
     }
@@ -531,7 +543,7 @@ sub source {
     while ($block = $sfd->readwindow) {
         foreach my $tag ($BAD_MULTILINE_CONSTRUCT_IN_RULES->all) {
             my $regex = $BAD_MULTILINE_CONSTRUCT_IN_RULES->value($tag);
-            $self->tag($tag) if $block =~ m/$regex/;
+            $self->hint($tag) if $block =~ m/$regex/;
         }
     }
     close($rules_fd);
